<?php
/**
 * Script to update list of supported languages.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * http://www.gnu.org/copyleft/gpl.html
 *
 * @file
 * @author Ori Livneh <ori@wikimedia.org>
 * @ingroup Maintenance
 */

$IP = getenv( 'MW_INSTALL_PATH' ) ?: __DIR__ . '/../../..';

require_once "$IP/maintenance/Maintenance.php";
require_once __DIR__ . "/../geshi/geshi.php";

class UpdateLanguageList extends Maintenance {
	public function __construct() {
		parent::__construct();
		$this->addDescription( 'Update list of languages supported by SyntaxHighlight_GeSHi' );
	}

	public function execute() {
		global $IP;

		$geshi = new GeSHi;
		$header = '// Generated by ' . basename( __FILE__ ) . ' on ' . date( DATE_RFC2822 ) . "\n";
		$langs = array_values( array_filter( $geshi->get_supported_languages( false ), 'ctype_alnum' ) );
		sort( $langs );
		$replace = array( '[' => "array(\n\t", ']' => "\n);\n", '",' => "\",\n\t" );
		$code = "<?php\n" .
			$header .
			"global \$wgGeSHiSupportedLanguages;\n" .
			'$wgGeSHiSupportedLanguages = ' . strtr( json_encode( $langs ), $replace );
		file_put_contents( __DIR__ . '/../SyntaxHighlight_GeSHi.langs.php', $code );
		$this->output( "Updated language list written to SyntaxHighlight_GeSHi.langs.php\n" );
	}
}

$maintClass = "UpdateLanguageList";
require_once( RUN_MAINTENANCE_IF_MAIN );
